/***************************************************************************
 *   Copyright (C) 2010 by Vladimir Jary                                   *
 *   <Vladimir.Jary@cern.ch>                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "speexporter.h"

#include<QtCore/QFile>
#include<QtCore/QString>
#include<QtCore/QIODevice>
#include<QtCore/QDataStream>
#include<QtCore/QTextStream>
#include<QtCore/QStringList>

#include "iostream"


/*
  Initialization of input and output streams and parses
  header of input SPE file
*/
SpeExporter::SpeExporter(QString input)
{
    inFile = new QFile();
    outFile = new QFile();
    inputName = input;
    inFile->setFileName(input);
    inFile->open(QIODevice::ReadOnly);
    in = new QDataStream(inFile);
    out = new QTextStream;
    parseHeader();
}

/*
  Closes and deletes specified file
*/
void SpeExporter::deleteFile(QFile *file)
{
    if (file){
        if (file->isOpen()) file->close();
        delete file;
        file = 0;
    }
}

/*
   Clean-up, destroys files
 */
SpeExporter::~SpeExporter()
{
    deleteFile(inFile);
    deleteFile(outFile);
}

/*
  Processes header of opened SPE file
  x ... stored at position 42
  y ... stored at position 656
*/
void SpeExporter::parseHeader()
{
    in->skipRawData(42);
    x = readWord(in);

    in->skipRawData(612);
    y = readWord(in);

    size = x*y;
    //skip rest of header
    in->skipRawData(3442);
}

/*
  Reads unsigned 16bit integer from input data stream
 */
quint16 SpeExporter::readWord(QDataStream* instream)
{
    qint8 byte1, byte2;
    qint32 b1, b2;
    qint16 word;
    instream->setByteOrder(QDataStream::LittleEndian);
    instream->operator >>(byte1);
    instream->operator >>(byte2);

    b1 = byte1;
    b2 = byte2;
    if (b1 < 0) b1+=256;
    if (b2 < 0) b2+=256;
    word = (b2 << 8) | b1;
    return word;
}


/*
  Prepares input and output streams
 */
void SpeExporter::openStreams()
{
    in->setDevice(inFile);
    outFile->setFileName(outputName);
    outFile->open(QIODevice::WriteOnly);
    out->setDevice(outFile);
}

/*
  Closes opened files
 */
void SpeExporter::closeStreams()
{
    inFile->close();
    outFile->close();
}

/*
  Converts SPE file into plain-text format
 */
void SpeExporter::export2txt()
{
    constructOutputName(".txt");
    openStreams();
    quint16 intensity;
    for(qint32 j = 0; j < size; j++){
        intensity = readWord(in);
        out->operator << (intensity);
        out->operator << ('\n');
    }
    closeStreams();
}

/*
  Converts input SPE file into comma-separated value
  spreadsheet
 */
void SpeExporter::export2csv()
{
    constructOutputName(".csv");
    openStreams();
    quint16 intensity;

    for(qint32 j = 1; j <= size; j++){
        intensity = readWord(in);
        out->operator << (j);
        out->operator << (',');
        out->operator << (intensity);
        out->operator << ('\n');
    }
    closeStreams();
}


/*
  Converts input SPE file into portable graymap
 */
void SpeExporter::export2pgm()
{
    constructOutputName(".pgm");
    openStreams();
    quint16 intensity;
    //max pixel intensity
    quint16 max = 0;
    QString line = "";
    //image data
    QStringList lines;

    for(qint32 j = 1; j <= size; j++){
        intensity = readWord(in);
        if (intensity > max)
            max = intensity;
        line.append(QString("%1").arg(intensity));
        if(line.length() < 65)
            line.append(' ');
        else
        {
            line.append('\n');
            lines.push_back(line);
            line = "";
        }

    }
    lines.push_back(line);

    //write header
    out->operator <<("P2\n");
    out->operator << (x) << (' ') << (y) << ('\n');
    out->operator <<(max) << ("\n");

    //write data
    QStringListIterator it(lines);
    while(it.hasNext())
    {
        out->operator <<(it.next());
    }

    lines.clear();
    closeStreams();
}

/*
  Generates name for output file - same as input, but in lowercase
  and with changed extension (SPE -> csv/pgm/txt)
 */
void SpeExporter::constructOutputName(QString extension)
{
  outputName = inputName.toLower();
  outputName.replace(".spe", extension);
}

/*
  Returns width in pixels
 */
inline qint16 SpeExporter::getX()
{
    return x;
}

/*
  Returns height in pixels
 */
inline qint16 SpeExporter::getY()
{
    return y;
}
